from zope.interface import implements
from zope.schema import Object

from plone.namedfile.interfaces import HAVE_BLOBS

from plone.namedfile.interfaces import INamedFileField, INamedImageField

from plone.namedfile.interfaces import INamedFile, INamedImage
from plone.namedfile.file import NamedFile as FileValueType
from plone.namedfile.file import NamedImage as ImageValueType


if HAVE_BLOBS:
    from plone.namedfile.interfaces import INamedBlobFileField, INamedBlobImageField
    from plone.namedfile.interfaces import INamedBlobFile, INamedBlobImage
    
    from plone.namedfile.file import NamedBlobFile as BlobFileValueType
    from plone.namedfile.file import NamedBlobImage as BlobImageValueType

class NamedFile(Object):
    """A NamedFile field
    """
    implements(INamedFileField)
    
    _type = FileValueType
    schema = INamedFile
    
    def __init__(self, **kw):
        if 'schema' in kw:
            self.schema = kw.pop('schema')
        super(NamedFile, self).__init__(schema=self.schema, **kw)

class NamedImage(Object):
    """A NamedImage field
    """
    implements(INamedImageField)
    
    _type = ImageValueType
    schema = INamedImage
    
    def __init__(self, **kw):
        if 'schema' in kw:
            self.schema = kw.pop('schema')
        super(NamedImage, self).__init__(schema=self.schema, **kw)

if HAVE_BLOBS:

    class NamedBlobFile(Object):
        """A NamedBlobFile field
        """
        implements(INamedBlobFileField)
    
        _type = BlobFileValueType
        schema = INamedBlobFile
    
        def __init__(self, **kw):
            if 'schema' in kw:
                self.schema = kw.pop('schema')
            super(NamedBlobFile, self).__init__(schema=self.schema, **kw)

    class NamedBlobImage(Object):
        """A NamedBlobImage field
        """
        implements(INamedBlobImageField)
    
        _type = BlobImageValueType
        schema = INamedBlobImage
    
        def __init__(self, **kw):
            if 'schema' in kw:
                self.schema = kw.pop('schema')
            super(NamedBlobImage, self).__init__(schema=self.schema, **kw)


import unittest
from zope.testing import doctestunit
from zope.component import testing

def test_suite():
    return unittest.TestSuite([

        doctestunit.DocFileSuite(
            'usage.txt', package='plone.namedfile',
            setUp=testing.setUp, tearDown=testing.tearDown),
        
        doctestunit.DocFileSuite(
            'handler.txt', package='plone.namedfile',
            setUp=testing.setUp, tearDown=testing.tearDown),
        
        doctestunit.DocFileSuite(
            'marshaler.txt', package='plone.namedfile',
            setUp=testing.setUp, tearDown=testing.tearDown),
        
        ])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



from zope.interface import implements

from zope.app.file.file import File
from zope.app.file.image import Image

from plone.namedfile.interfaces import HAVE_BLOBS

from plone.namedfile.interfaces import INamedFile, INamedImage
from plone.namedfile.utils import get_contenttype

if HAVE_BLOBS:
    from z3c.blobfile.file import File as BlobFile
    from z3c.blobfile.image import Image as BlobImage
    from plone.namedfile.interfaces import INamedBlobFile, INamedBlobImage

class NamedFile(File):
    """A non-BLOB file that stores a filename
    """
    implements(INamedFile)

    def __init__(self, data='', contentType='', filename=None):
        if filename is not None and contentType in ('', 'application/octet-stream'):
            contentType = get_contenttype(filename=filename)
        super(NamedFile, self).__init__(data, contentType)
        self.filename = filename

class NamedImage(Image):
    """An non-BLOB image with a filename
    """
    implements(INamedImage)

    def __init__(self, data='', contentType='', filename=None):
        super(NamedImage, self).__init__(data)
        self.filename = filename
        
        # Allow override of the image sniffer
        if contentType:
            self.contentType = contentType

if HAVE_BLOBS:

    class NamedBlobFile(BlobFile):
        """A file stored in a ZODB BLOB, file a filename
        """
        implements(INamedBlobFile)

        def __init__(self, data='', contentType='', filename=None):
            if filename is not None and contentType in ('', 'application/octet-stream'):
                contentType = get_contenttype(filename=filename)
            super(NamedBlobFile, self).__init__(data, contentType)
            self.filename = filename

    class NamedBlobImage(BlobImage):
        """An image stored in a ZODB BLOB with a filename
        """
        implements(INamedBlobImage)

        def __init__(self, data='', contentType='', filename=None):
            super(NamedBlobImage, self).__init__(data)
            self.filename = filename
        
            # Allow override of the image sniffer
            if contentType:
                self.contentType = contentType



from zope.interface import Interface

from zope import schema
from zope.schema.interfaces import IObject

from zope.app.file.interfaces import IFile, IImage

try:
    from z3c.blobfile.interfaces import IBlobFile, IBlobImage
    HAVE_BLOBS = True
except ImportError:
    HAVE_BLOBS = False

# Values

class INamed(Interface):
    """An item with a filename
    """
    
    filename = schema.TextLine(title=u"Filename", required=False, default=None)

class INamedFile(INamed, IFile):
    """A non-BLOB file with a filename
    """

class INamedImage(INamed, IImage):
    """A non-BLOB image with a filename
    """

# Fields

class INamedField(IObject):
    """Base field type
    """

class INamedFileField(INamedField):
    """Field for storing INamedFile objects.
    """

class INamedImageField(INamedField):
    """Field for storing INamedImage objects.
    """

if HAVE_BLOBS:

    # Values

    class IBlobby(Interface):
        """Marker interface for objects that support blobs.
        """
    
    class INamedBlobFile(INamedFile, IBlobby, IBlobFile):
        """A BLOB file with a filename
        """

    class INamedBlobImage(INamedImage, IBlobby, IBlobImage):
        """A BLOB image with a filename
        """
        
    # Fields

    class INamedBlobFileField(INamedFileField):
        """Field for storing INamedBlobFile objects.
        """

    class INamedBlobImageField(INamedImageField):
        """Field for storing INamedBlobImage objects.
        """



from plone.namedfile.interfaces import HAVE_BLOBS
from plone.namedfile import field
from plone.schemaeditor.fields import FieldFactory

NamedFileFactory = FieldFactory(field.NamedFile, u'File Upload')
NamedImageFactory = FieldFactory(field.NamedImage, u'Image')

if HAVE_BLOBS:
    NamedBlobFileFactory = FieldFactory(field.NamedBlobFile, u'File Upload')
    NamedBlobImageFactory = FieldFactory(field.NamedBlobImage, u'Image')



from zope.interface import implements
from zope.publisher.interfaces import IPublishTraverse, NotFound

from plone.namedfile.utils import set_headers, stream_data

from Acquisition import aq_base
from Products.Five.browser import BrowserView

class Download(BrowserView):
    """Download a file, via ../context/@@download/fieldname/filename
    
    `fieldname` is the name of an attribute on the context that contains
    the file. `filename` is the filename that the browser will be told to
    give the file. If not given, it will be looked up from the field.
    
    The attribute under `fieldname` should contain a named (blob) file/image
    instance from this package.
    """
    
    implements(IPublishTraverse)
    
    def __init__(self, context, request):
        super(Download, self).__init__(context, request)
        self.fieldname = None
        self.filename = None
        
    def publishTraverse(self, request, name):
        
        if self.fieldname is None:  # ../@@download/fieldname
            self.fieldname = name
        elif self.filename is None: # ../@@download/fieldname/filename
            self.filename = name
        else:
            raise NotFound(self, name, request)
        
        return self
    
    def __call__(self):
        
        # Ensure that we have at least a filedname
        if not self.fieldname:
            raise NotFound(self, '', self.request)
        
        file = getattr(aq_base(self.context), self.fieldname, None)
        if file is None:
            raise NotFound(self, self.fieldname, self.request)
        
        if not self.filename:
            self.filename = getattr(file, 'filename', self.fieldname)
        
        set_headers(file, self.request.response, filename=self.filename)
        
        return stream_data(file)


try:
    from plone.supermodel.exportimport import ObjectHandler
    HAVE_SUPERMODEL = True
except ImportError:
    HAVE_SUPERMODEL = False
    
if HAVE_SUPERMODEL:

    from zope.interface import implements
    from zope.component import adapts
    
    from plone.namedfile import field
    from plone.namedfile.interfaces import HAVE_BLOBS
    
    class FileFieldHandler(ObjectHandler):
        
        filteredAttributes = ObjectHandler.filteredAttributes.copy()
        filteredAttributes.update({'default': 'rw', 'missing_value': 'rw', 'schema': 'rw'})
    
    NamedFileHandler = FileFieldHandler(field.NamedFile)
    NamedImageHandler = FileFieldHandler(field.NamedImage)
    
    if HAVE_BLOBS:
        NamedBlobFileHandler = FileFieldHandler(field.NamedBlobFile)
        NamedBlobImageHandler = FileFieldHandler(field.NamedBlobImage)



from setuptools import setup, find_packages
import os

version = '1.0b4'

setup(name='plone.namedfile',
      version=version,
      description="File types tand fields for images, files and blob files with filenames",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read() + "\n" + 
                       open(os.path.join("plone", "namedfile", "usage.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='plone named file image blob',
      author='Laurence Rowe, Martin Aspeli',
      author_email='plone-developers@lists.sourceforge.net',
      url='http://pypi.python.org/pypi/plone.namedfile',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['plone'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'zope.app.file',
      ],
      extras_require = {
        'blobs':  ['z3c.blobfile'],
        'editor': ['plone.schemaeditor'],
        'supermodel': ['plone.supermodel'],
        'marshaler': ['plone.rfc822'],
      },
      entry_points="""
      # -*- Entry points: -*-
      """,
      )
